/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.financial.erpcredit.service;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.audit.information.entity.Information;
import com.jeeplus.modules.sg.audit.information.service.InformationService;
import com.jeeplus.modules.sg.financial.erpcredit.entity.*;
import com.jeeplus.modules.sg.financial.erpcredit.mapper.ErpCreditEquipmentMapper;
import com.jeeplus.modules.sg.financial.erpcredit.mapper.ErpCreditMapper;
import com.jeeplus.modules.sg.financial.erpcredit.mapper.ErpCreditMaterialMapper;
import com.jeeplus.modules.sg.financial.erpcredit.util.ExportTemplate;
import com.jeeplus.modules.sg.financial.erpcredit.util.ExportUtil2;
import com.jeeplus.modules.sg.financial.settlement.entity.MaintainData;
import com.jeeplus.modules.sg.financial.settlement.mapper.DataMaintenanceMapper;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;


@Service
@Transactional(readOnly = true)
public class ErpCreditService extends CrudService<ErpCreditMapper,ErpCredit> {
    @Autowired
    private ErpCreditEquipmentMapper erpCreditEquipmentMapper;
    @Autowired
    private ErpCreditMaterialMapper erpCreditMaterialMapper;
    @Autowired
    private ErpCreditMapper erpCreditMapper;
    @Autowired
    private DataMaintenanceMapper dataMaintenanceMapper;

    @Autowired
    private InformationService informationService;

    public List<ErpCredit> getListErpCredit(List<String> strings){
        List<ErpCredit> erpCreditList = erpCreditMapper.findErpCreditList(strings);
        List<ErpCreditEquipment> erpCreditChild1List = erpCreditEquipmentMapper.findErpCreditChild1List(strings);
        List<ErpCreditMaterial> erpCreditChild2List = erpCreditMaterialMapper.findErpCreditChild2List(strings);
        if (null!=erpCreditList&&erpCreditList.size()>0){
            for (ErpCredit erpCredit:erpCreditList){
                if (null!=erpCreditChild1List&&erpCreditChild1List.size()>0){
                    for (ErpCreditEquipment erpCreditEquipment:erpCreditChild1List){
                        if (erpCredit.getId().equals(erpCreditEquipment.getErpCredit().getId())){
                            erpCredit.getErpCreditChild1s().add(erpCreditEquipment);
                        }
                    }
                }
                if (null!=erpCreditChild2List&&erpCreditChild2List.size()>0){
                    for (ErpCreditMaterial erpCreditChild2:erpCreditChild2List){
                        if (erpCredit.getId().equals(erpCreditChild2.getErpCredit().getId())){
                            erpCredit.getErpCreditChild2s().add(erpCreditChild2);
                        }
                    }
                }
            }
        }
        return erpCreditList;
    }
    public ErpCredit get(String id) {
        ErpCredit erpCredit = super.get(id);
        erpCredit.setErpCreditChild1s(erpCreditEquipmentMapper.findList(new ErpCreditEquipment(erpCredit)));
        erpCredit.setErpCreditChild2s(erpCreditMaterialMapper.findList(new ErpCreditMaterial(erpCredit)));
//        erpCredit.setErpCreditChild3s(erpCreditChild3Mapper.findList(new ErpCreditChild3(erpCredit)));
        return erpCredit;
    }

    public List<ErpCredit> findList(ErpCredit erpCredit) {
        return super.findList(erpCredit);
    }

    public Page<ErpCredit> findPage(Page<ErpCredit> page, ErpCredit erpCredit) {
        return super.findPage(page, erpCredit);
    }

    @Transactional(readOnly = false)
    public void save(ErpCredit erpCredit) {
        super.save(erpCredit);
        for (ErpCreditEquipment erpCreditChild1 : erpCredit.getErpCreditChild1s()){
            if (ErpCreditEquipment.DEL_FLAG_NORMAL.equals(erpCredit.getDelFlag())){
                if (StringUtils.isBlank(erpCreditChild1.getId())) {
                    erpCreditChild1.setErpCredit(erpCredit);
                    erpCreditChild1.preInsert();
                    erpCreditEquipmentMapper.insert(erpCreditChild1);
                }else {
                    erpCreditChild1.preUpdate();
                    erpCreditEquipmentMapper.update(erpCreditChild1);
                }
            }else {
                erpCreditEquipmentMapper.delete(erpCreditChild1);
            }
        }
        for (ErpCreditMaterial erpCreditChild2 : erpCredit.getErpCreditChild2s()){
            if (ErpCreditMaterial.DEL_FLAG_NORMAL.equals(erpCreditChild2.getDelFlag())){
                if (StringUtils.isBlank(erpCreditChild2.getId())){
                    erpCreditChild2.setErpCredit(erpCredit);
                    erpCreditChild2.preInsert();
                    erpCreditMaterialMapper.insert(erpCreditChild2);
                }else {
                    erpCreditChild2.preUpdate();
                    erpCreditMaterialMapper.update(erpCreditChild2);
                }
            }else {
                erpCreditMaterialMapper.delete(erpCreditChild2);
            }
        }
//        for (ErpCreditChild3 erpCreditChild3 : erpCredit.getErpCreditChild3s()){
//            if (ErpCreditChild3.DEL_FLAG_NORMAL.equals(erpCreditChild3.getDelFlag())){
//                if (StringUtils.isBlank(erpCreditChild3.getId())){
//                    erpCreditChild3.setErpCredit(erpCredit);
//                    erpCreditChild3.preInsert();
//                    erpCreditChild3Mapper.insert(erpCreditChild3);
//                }else {
//                    erpCreditChild3.preUpdate();
//                    erpCreditChild3Mapper.update(erpCreditChild3);
//                }
//            }else {
//                erpCreditChild3Mapper.delete(erpCreditChild3);
//            }
//        }
    }

    @Transactional(readOnly = false)
    public void delete(ErpCredit erpCredit) {
        super.delete(erpCredit);
        erpCreditEquipmentMapper.delete(new ErpCreditEquipment(erpCredit));
        erpCreditMaterialMapper.delete(new ErpCreditMaterial(erpCredit));
//        erpCreditChild3Mapper.delete(new ErpCreditChild3(erpCredit));
    }

    //维护表
    public List<MaintainData> getFindListMain(List<String> strings){
        return dataMaintenanceMapper.getFindListMain(strings);
    }

    //导出
    public void getExport(List<String> strings, HttpServletResponse response){
        List<ErpCredit> erpCreditList = getListErpCredit(strings);
        List<MaintainData> maintainDatas = getFindListMain(strings);
        ExportTemplate exportTemplate = new ExportTemplate();
        File newFile = exportTemplate.createNewFile();
        // 新文件写入数据，并下载
        InputStream inputStream = null;
        XSSFWorkbook workbook = null;
        XSSFFont font=null;
        XSSFSheet sheet1 = null;
        XSSFSheet sheet2 = null;
        XSSFSheet sheet3 = null;
        XSSFSheet sheet5 = null;
        XSSFSheet sheet4 = null;
        XSSFSheet sheet6=null;
        XSSFSheet sheet7=null;
        XSSFSheet sheet8=null;
        XSSFSheet sheet10=null;
        XSSFSheet sheet11=null;
        try {
            inputStream = new FileInputStream(newFile);// 将excel文件转为输入流
            workbook = new XSSFWorkbook(inputStream);// 创建个workbook，
            font = workbook.createFont();
            sheet1 =workbook.getSheetAt(1);
            sheet2 = workbook.getSheetAt(2);
            sheet3 = workbook.getSheetAt(3);
            sheet4 = workbook.getSheetAt(4);
            sheet5=workbook.getSheetAt(5);
            sheet6=workbook.getSheetAt(6);
            sheet7=workbook.getSheetAt(7);
            sheet8=workbook.getSheetAt(8);
            sheet10=workbook.getSheetAt(10);
            sheet11=workbook.getSheetAt(11);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        try {
            // 写数据
            FileOutputStream fos = new FileOutputStream(newFile);
            ExportUtil2.getSheet1(sheet1,erpCreditList,maintainDatas);
            ExportUtil2.getSheet2(sheet2,erpCreditList,maintainDatas);
            ExportUtil2.getSheet3(sheet3,erpCreditList,maintainDatas);
            ExportUtil2.getSheet4(sheet4,erpCreditList);
            ExportUtil2.getSheet5(sheet5,erpCreditList,workbook);
            ExportUtil2.getSheet6(sheet6,erpCreditList,workbook);
            ExportUtil2.getSheet7(sheet7,erpCreditList);
            ExportUtil2.getSheet8(sheet8,erpCreditList,maintainDatas);
            ExportUtil2.getSheet10(sheet10,erpCreditList,workbook);
            ExportUtil2.getSheet11(sheet11,erpCreditList,workbook);
            workbook.write(fos);
            fos.flush();
            fos.close();
            InputStream fis = new BufferedInputStream(new FileInputStream(
                    newFile));
            byte[] buffer = new byte[fis.available()];
            fis.read(buffer);
            fis.close();
            response.reset();
            response.setContentType("text/html;charset=UTF-8");
            OutputStream toClient = new BufferedOutputStream(response.getOutputStream());
            response.setContentType("application/x-msdownload");
            String newName = URLEncoder.encode("竣工结算表" + System.currentTimeMillis() + ".xlsx","UTF-8");
            response.addHeader("Content-Disposition","attachment;filename=\"" + newName + "\"");
            response.addHeader("Content-Length", "" + newFile.length());
            toClient.write(buffer);
            toClient.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (null != inputStream) {
                    inputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        // 删除创建的新文件
        // this.deleteFile(newFile);
    }

    /**
     * 判断
     * @return
     */
    public List getCheck(String ids){
        List<String> lists = new ArrayList<String>();
        boolean flag = true;
        String[] idArray =ids.split(",");
        for (String str:idArray){
            lists.add(str);
        }
//        List<MaintainData> maintainDatas = getFindListMain(lists);
        List<Information> informationList = informationService.findNoFinishList(lists);
//        if (null!=informationList&&informationList.size()>0){
//            for (int k = 0; k < lists.size(); k++) {
//                flag = true;
//                for(Information information:informationList){
//                    if (information.getProjectId().equals(lists.get(k))){
//                        flag=false;
//                        break;
//                    }
//                }
//                if (flag){
//                    lists.remove(k);
//                    k--;
//                }
//            }
//        }else {
//            lists=new ArrayList<String>();
//        }
//        if (null!=maintainDatas&&maintainDatas.size()>0){
//             for (int i = 0; i < lists.size(); i++) {
//                flag=true;
//                for (MaintainData maintainData:maintainDatas){
//                    if (maintainData.getProjectId().equals(lists.get(i))){
//                        flag=false;
//                        break;
//                    }
//                }
//                if (flag){
//                    lists.remove(i);
//                    i--;
//                }
//             }
//        }else {
//            lists=null;
//        }
        return lists;
    }
}